const db = require("../database/db");
const sharp = require("sharp");
const { S3Client, PutObjectCommand } = require('@aws-sdk/client-s3');
const { sendMessage } = require("../../util/gzappy");
const cron = require("node-cron");
const { agendarMensagemPromocional } = require("../../util/cron");


require("dotenv").config({ path: __dirname + "/./../../../.env" });

class mensagensRepository {

    constructor() {
        this.db = db;
        this.s3Client = new S3Client({
            region: process.env.S3_REGION,
            credentials: {
              accessKeyId: process.env.AWS_ACCESS_KEY,
              secretAccessKey: process.env.AWS_SECRET_KEY,
            },
          });
    }


    /**
     * Registra uma mensagem
     * @param {*} data dados de uma mensagem
     * @returns mensagem referente ao resultado obtido
     */
    register(data, file) {
        return new Promise(async (resolve, reject) => {
            try {
                if (file) {
                    const buffer = await sharp(file.buffer).resize({ fit: "contain" }).toBuffer()
                    const command = new PutObjectCommand({
                        Bucket: process.env.S3_BUCKET,
                        Key: file.originalname,
                        Body: buffer,
                        ContentType: file.mimetype
                    });
                    await this.s3Client.send(command);
                }

                data.padrao = (data.horario == null && data.data == null) ? true : false;

                this.db.query(
                    `INSERT INTO mensagens (mensagem, data, hora, assunto,  padrao, image) VALUES (?,?,?,?,?,?)`,
                    [data.mensagem, data?.data, data?.hora, data.assunto, data.padrao, file ? "https://aliancapedidos.s3.us-east-2.amazonaws.com/"+file.originalname : ""],
                    async (error, response) => {
                        if (error) {
                            console.error("Erro ao inserir mensagem:", error);
                            return reject(error);
                        }
                        return resolve({ success: "Mensagem cadastrada com sucesso!", code: 200 });
                    }
                );

                if (process.env.NODE_ENV === "production") {
                    if (!data.padrao) {
                        agendarMensagemPromocional({
                            mensagem: data.mensagem,
                            data: data.data,
                            hora: data.hora,
                            image: file ? "https://aliancapedidos.s3.us-east-2.amazonaws.com/" + file.originalname : ""
                        });
                    }
                }
        

            } catch (error) {
                console.error("Erro na operação register:", error);
                return reject(error);
            }
        });
    }


    /**
     * Lista todos os distribuidores
     * @returns lista com todos os dados de um distribuidor
     */
    listMensagensVinculadas(id_mensagem) {
        return new Promise(async (resolve, reject) => {


            try {

                this.db.query(
                    `SELECT mensagens.id, mensagens_vinculadas.id_user, user.razao_social
                             FROM mensagens, mensagens_vinculadas, user
                             WHERE
                             mensagens_vinculadas.id_mensagem = mensagens.id AND
                             mensagens_vinculadas.id_user = user.id AND
                             mensagens.id = ?`,
                    [id_mensagem ? id_mensagem : ""],
                    async (error, mensagens) => {
                        if (error) {
                            console.log(error);
                            return reject(new Error(error));
                        }

                        // console.log(mensagens.image);
                        // console.log(mensagem.image.split('/')[2]);
                        


                        // for (const mensagem of mensagens) {
                        //     if (mensagem.image.split('/')[2] == "null") mensagem = ""
                            
                        //   }

                          
                          console.log(mensagens.image);


                        if (mensagens.length === 0) return resolve({ id: id_mensagem, usuarios: [] });
                        

                        if (mensagens.length === 1) {
                            const mensagem = mensagens[0];
                            return resolve({
                                id: id_mensagem,
                                usuarios: [{ id: mensagem.id_user, razao_social: mensagem.razao_social }]
                            });
                        }

                        const result = [];
                        let existingMessage

                        mensagens.forEach(mensagem => {
                            existingMessage = result.find(item => item.mensagem === mensagem.mensagem);


                            if (existingMessage) {
                                if (
                                    !existingMessage.usuarios.includes({
                                        id: mensagem.id_user,
                                        razao_social: mensagem.razao_social,
                                    })
                                ) {
                                    existingMessage.usuarios.push({
                                        id: mensagem.id_user,
                                        razao_social: mensagem.razao_social,
                                    });
                                }
                            }

                            else {
                                result.push({
                                    id: mensagem.id,
                                    usuarios: [{ id: mensagem.id_user, razao_social: mensagem.razao_social }],
                                });
                            }
                        });
                        return resolve(existingMessage);
                    }
                );


            } catch (error) {
                console.error(error);
                return reject(new Error(error));
            };
        })
    }


    /**
     * Lista todos as mensagens
     * @returns lista com todas as mensagens
     */
    listMensagens(id) {
        return new Promise(async (resolve, reject) => {
            var query = id
                ? `WHERE id_user = ?`
                : ``

            try {

                this.db.query(`SELECT mensagens.id, mensagens.mensagem, DATE_FORMAT(mensagens.data, "%d/%m/%Y") AS data, mensagens.padrao,
                    mensagens.hora, mensagens.assunto, mensagens.image
                    FROM mensagens
                     ${query}`, [id ? id : ""],
                    async (error, mensagens) => {
                        if (error) {
                            console.error(error);
                            return reject(new Error(error));
                        }

                        // for (let mensagem of mensagens) {

                        //     let url 

                        //     mensagem.image == null ? url = "" : url = "https://alianca.s3.us-east-2.amazonaws.com/"+mensagem.image
                        //     mensagem.image = url
                        //   }

                          return resolve(mensagens);
                        });

            } catch (error) {
                console.error(error);
                return reject(new Error(error));
            }
        });
    }

    /**
     * Lista todos os  mensagens
     * @returns lista com todas as mensagens
     */
      listAssuntoMensagens(id) {
        return new Promise(async (resolve, reject) => {


            try {

                this.db.query(`SELECT mensagens.id, mensagens.assunto FROM mensagens WHERE padrao = true
                     `,[],
                    async (error, mensagens) => {
                        if (error) {
                            console.error(error);
                            return reject(new Error(error));
                        }

                        return resolve(mensagens);
                        });

            } catch (error) {
                console.error(error);
                return reject(new Error(error));
            }
        });
    }


    /**
     * Deleta um distribuidor
     * @param {*} id Id do distribuidor a ser excluído
     * @returns mensagem referente ao resultado obtido
     */
    delete(id) {
        return new Promise((resolve, reject) => {

            this.db.query('DELETE FROM mensagens WHERE id = ?', [id],
                (error, response) => {
                    if (error) {
                        return resolve({ error: "Esta pedidos possui vinculos!", code: 203 });
                    }
                    else {
                        return resolve(true);
                    }
                });
        });
    }

    desvincularMensagem(data) {
        return new Promise((resolve, reject) => {

            this.db.query('DELETE FROM mensagens_vinculadas WHERE id_user = ? AND id_mensagem = ? AND horario = ?', [data.id_user, data.id_mensagem, data.horario],
                (error, response) => {
                    if (error) {
                        console.error(error);
                        return resolve({ error: "Esta mensagem possui vinculos!", code: 203 });
                    }
                    else {
                        return resolve(true);
                    }
                });
        });
    }


    /**
     * Atualiza os dados de uma mensagem 
     * @param {*} data dados atualizados da mensagem
     * @returns mensagem referente ao resultado obtido
     */
    update(data, file) {
        return new Promise(async (resolve, reject) => {
            try {
                let imageUrl = "";
    
                if (file) {
                    const buffer = await sharp(file.buffer).resize({ fit: "contain" }).toBuffer();
                    const command = new PutObjectCommand({
                        Bucket: process.env.S3_BUCKET,
                        Key: file.originalname,
                        Body: buffer,
                        ContentType: file.mimetype
                    });
                    await this.s3Client.send(command);
                    imageUrl = "https://aliancapedidos.s3.us-east-2.amazonaws.com/" + file.originalname;
                }
    
                data.padrao = (data.horario == null && data.data == null) ? true : false;
    
                this.db.query(
                    "UPDATE mensagens SET mensagem=?, data=?, hora=?, padrao=?, image=? WHERE id = ?",
                    [data.mensagem, data?.data, data?.hora, data?.padrao, imageUrl, data.id],
                    (error, response) => {
                        if (error) return reject(new Error(error));
    
                        if (process.env.NODE_ENV === "production") {
                            if (!data.padrao) {
                                agendarMensagemPromocional({
                                    mensagem: data.mensagem,
                                    data: data.data,
                                    hora: data.hora,
                                    image: imageUrl // importante para o sendImage funcionar
                                });
                            }
                        }
    
                        return resolve({ success: "Mensagem atualizada com sucesso!", code: 200 });
                    }
                );
            } catch (error) {
                console.error("Erro ao atualizar mensagem:", error);
                return reject(error);
            }
        });
    }
    

    /**
     * Atualiza os dados de uma mensagem 
     * @param {*} data dados atualizados da mensagem
     * @returns mensagem referente ao resultado obtido
     */
    vincularUsuarioMensangem(data) {
        return new Promise(async (resolve, reject) => {


            if (!Array.isArray(data.id_user)) {
                return reject(new Error("O campo id_user deve ser um array."));
            }
            const x = data.id_user.length
            for (let index = 0; index < x; index++) {
                this.db.query(
                    "INSERT INTO mensagens_vinculadas (id_user, id_mensagem) VALUES (?,?)",
                    [data.id_user[index], data.id_mensagem],
                    (error, response) => {
                        if (error) return reject(error);
                        else return resolve(response);
                    }
                );
            }


            return resolve({ success: "Mensagem vinculada com sucesso!", code: 200 });
        });
    }

    /**
     * Atualiza os dados de uma mensagem 
     * @param {*} data dados atualizados da mensagem
     * @returns mensagem referente ao resultado obtido
     */
    enviarLembreteParaUsuario(data) {
        return new Promise(async (resolve, reject) => {




            const usuario = await new Promise((resolve, reject) => {
                this.db.query(
                  `SELECT * FROM user WHERE id = ?`,
                  [data.id_user],
                  (error, results) => {
                    if (error) {
                      console.error("Error fetching existing order:", error);
                      return reject(error);
                    }
                    resolve(results[0]);
                  }
                );
              });

              const lembrete_instantaneo = await new Promise((resolve, reject) => {
                this.db.query(
                  `SELECT * FROM lembrete_instantaneo WHERE id_user = ? AND data_envio = CURDATE()`,
                  [data.id_user],
                  (error, results) => {
                    if (error) {
                      console.error("Error fetching existing order:", error);
                      return reject(error);
                    }
                    resolve(results[0]);
                  }
                );
              });

              if (lembrete_instantaneo?.id_user === usuario?.id) return resolve({ error: `Lembrete ja foi enviado para esse usuario`, code: 203 })



              const messages = await new Promise((resolve, reject) => {
                db.query(
                  `SELECT mensagem FROM mensagens WHERE id = 83`,
                  [],
                  (error, results) => {
                    if (error) {
                      console.error("Error fetching existing order:", error);
                      return reject(error);
                    }
                    resolve(results[0].mensagem);
                  }
                );
              });



            this.db.query(
                "INSERT INTO lembrete_instantaneo (id_user, data_envio) VALUES (?,?)",
                [data.id_user, new Date()],
                (error, response) => {
                    if (error) return reject(error);
                    else return resolve(response);
                }
            );

            await sendMessage(messages, "+55"+usuario.celular)
            


            return resolve({ success: "Lembrete enviado com sucesso! ", code: 200 });
        });
    }

}
module.exports = mensagensRepository;